@@ -2,7 +2,7 @@
 #[test]
 fn test_() {
     use super::*;
-    const TEST_CASES: [(i64, &'static str); 1912] = [
+    const TEST_CASES: [(i64, &str); 1912] = [
         (21_4748_3647, "二一億四七百四八万三六百四七"),
         (21_4748_3647, "二十一億四七百四八万三六百四七"),
         (87_6543, "八十七万六五百四三"),
